@@ -19,7 +19,7 @@ from tornado.web import RedirectHandler
 from jupyter_core.application import JupyterApp, NoStart
 
 from jupyter_server.serverapp import ServerApp
-from jupyter_server.transutils import _
+from jupyter_server.transutils import _i18n
 from jupyter_server.utils import url_path_join
 from .handler import ExtensionHandlerMixin
 
@@ -87,7 +87,7 @@ class ExtensionAppJinjaMixin(HasTraits):
     """Use Jinja templates for HTML templates on top of an ExtensionApp."""
 
     jinja2_options = Dict(
-        help=_("""Options to pass to the jinja2 environment for this
+        help=_i18n("""Options to pass to the jinja2 environment for this
         """)
     ).tag(config=True)
 
@@ -154,7 +154,6 @@ class ExtensionApp(JupyterApp):
     # the default value to False if they don't offer a browser
     # based frontend.
     open_browser = Bool(
-        True,
         help="""Whether to open in a browser after starting.
         The specific browser used is platform dependent and
         determined by the python standard library `webbrowser`
@@ -163,6 +162,10 @@ class ExtensionApp(JupyterApp):
         """
     ).tag(config=True)
 
+    @default('open_browser')
+    def _default_open_browser(self):
+        return self.serverapp.config["ServerApp"].get("open_browser", True)
+
     # The extension name used to name the jupyter config
     # file, jupyter_{name}_config.
     # This should also match the jupyter subcommand used to launch
@@ -230,24 +233,24 @@ class ExtensionApp(JupyterApp):
     ).tag(config=True)
 
     template_paths = List(Unicode(),
-        help=_("""Paths to search for serving jinja templates.
+        help=_i18n("""Paths to search for serving jinja templates.
 
         Can be used to override templates from notebook.templates.""")
     ).tag(config=True)
 
     settings = Dict(
-        help=_("""Settings that will passed to the server.""")
+        help=_i18n("""Settings that will passed to the server.""")
     ).tag(config=True)
 
     handlers = List(
-        help=_("""Handlers appended to the server.""")
+        help=_i18n("""Handlers appended to the server.""")
     ).tag(config=True)
 
     def _config_file_name_default(self):
         """The default config file name."""
         if not self.name:
             return ''
-        return 'jupyter_{}_config'.format(self.name.replace('-','_'))
+        return 'jupyter_{}_config'.format(self.name.replace('-', '_'))
 
     def initialize_settings(self):
         """Override this method to add handling of settings."""
@@ -476,6 +479,7 @@ class ExtensionApp(JupyterApp):
         """
         serverapp = ServerApp.instance(
             jpserver_extensions={cls.get_extension_package(): True}, **kwargs)
+        serverapp.aliases.update(cls.aliases)
         serverapp.initialize(
             argv=argv,
             starter_extension=cls.name,
@@ -519,4 +523,4 @@ class ExtensionApp(JupyterApp):
         try:
             serverapp.start()
         except NoStart:
-            pass
\ No newline at end of file
+            pass
